// ARCCheck.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file ARCCheck.cs
 * Provide functions for check propriety of .arc file.*/

using System;

namespace Grubng
{   
	/// <summary>
	/// Provide function for check propriety of .arc file.
	/// </summary>
    internal static class ARCCheck
    {
		/// <summary>
		/// Function check propriety of .arc file. Check sequence of url's, length of each entry in file and type of each url.
		/// </summary>
		/// <param name="workunit">
		/// A <see cref="System.String"/> full path (including filename) to workunit
		/// </param>
		/// <param name="arcunit">
		/// A <see cref="System.String"/> full path (including filename) to .arc file to check
		/// </param>
		/// <param name="itermax">
		/// A <see cref="System.Int32"/> amount of url's in file (must be that same for both files)
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if all is ok, false if .arc file is corrupted.
		/// </returns>
        public static bool CheckFile(string workunit, string arcunit, int itermax)
        {
			if (!System.IO.File.Exists(arcunit))
			{
				return false;
			}
            bool correct = true;
			System.Text.StringBuilder work1 = new System.Text.StringBuilder();
			System.Text.StringBuilder work2 = new System.Text.StringBuilder();
			System.Text.StringBuilder urli = new System.Text.StringBuilder();
			System.Text.StringBuilder arc = new System.Text.StringBuilder();
			string[] arcfields = new string[5];
			string[] testurli = new string[3]; 
			int iter = 1;
			System.Collections.Generic.List<byte> tmp2 = new System.Collections.Generic.List<byte>();
			int tmp;
			using (System.IO.FileStream arcfile = System.IO.File.OpenRead(arcunit))
			{
				using (System.IO.StreamReader workstream = new System.IO.StreamReader(workunit))
				{
					arcfile.Position = 131;
					while (!workstream.EndOfStream)
					{
						work1.Append(workstream.ReadLine());
						while (true)
						{
							if (work1.Length > 0)
							{
								if (work1.ToString().Substring(0, 3) == "GET")
								{
									work2.Append(workstream.ReadLine());
									break;
								}
							}
							work1.Remove(0, work1.Length);
							work1.Append(workstream.ReadLine());
						}
						testurli = work1.ToString().Split(' ');
						work2 = work2.Replace("Host: ", String.Empty);
						urli.Append("http://");
						urli.Append(work2.ToString());
						urli.Append(testurli[1]);
						while (true)
						{
							tmp = arcfile.ReadByte();
							if ((tmp > Char.MaxValue) || (tmp < Char.MinValue))
							{
								correct = false;
								break;
							}
							if (Convert.ToChar(tmp) == '\n')
							{
								break;
							}
							tmp2.Add(Convert.ToByte(tmp));
						}
						if (!correct)
						{
							break;
						}
						arc.Append(System.Text.Encoding.UTF8.GetString(tmp2.ToArray(), 0, tmp2.ToArray().Length));
						tmp2.Clear();
						if (arc.Length == 0)
						{
							correct = false;
							break;
						}
						arcfields = arc.ToString().Split(' ');
						if (urli.ToString() != arcfields[0])
						{
							correct = false;
							break;
						}
						urli.Remove(0, urli.Length);
						if (String.IsNullOrEmpty(arcfields[3]))
						{
							correct = false;
							break;
						}
						arcfile.Position += Convert.ToInt64(arcfields[4]) + 1;
						iter++;
						work1.Remove(0, work1.Length);
						work2.Remove(0, work2.Length);
						arc.Remove(0, arc.Length);
						if (iter > itermax)
						{
							break;
						}
					}
					workstream.Close();
				}
				arcfile.Close();
			}
            return correct;
        }
    }
}
